@@ -2,8 +2,8 @@
 Implemenation of same basic algorithms used in cryptography.
 """
 
-from .nt import cf, convergents, jacobi_symbol, sqrt_mod, euler_phi, order, carmichael_lambda
-from .primes import sieve_eratosthenes, isprime
+from .nt import egcd, crt, cf, convergents, legendre_symbol, jacobi_symbol, sqrt_mod, euler_phi, order, carmichael_lambda
+from .primes import sieve_eratosthenes, is_prime, next_prime, random_prime, random_strongprime, is_safeprime, random_safeprime
 from .factor import factorint
 from .dlp import dlog
 from .ec import EC_Weierstrass
