@@ -3,6 +3,8 @@ Linear algebra
 """
 
 from math import inf, sqrt, prod
+from numbers import Number
+
 
 class Matrix:
     """
@@ -61,8 +63,10 @@ class Matrix:
             else:
                 cols = range(self.cols)[j]
             return Matrix([[self.matrix[i][j] for j in cols] for i in rows])
-        i, j = divmod(item, self.cols)
-        return self.matrix[i][j]
+        if isinstance(item, int):
+            i, j = divmod(item, self.cols)
+            return self.matrix[i][j]
+        return Matrix([self.matrix[k // self.cols][k % self.cols] for k in range(self.cols * self.rows)[item]])
 
     def __setitem__(self, item, value):
         if isinstance(item, tuple):
@@ -147,12 +151,16 @@ class Matrix:
         return result
 
     def __add__(self, other) -> "Matrix":
-        if isinstance(other, Matrix) and other.cols == self.cols and other.rows == self.rows:
+        if isinstance(other, Matrix):
+            if other.cols != self.cols or other.rows != self.rows:
+                raise NotImplementedError("Matrix dimensions do not match!")
             return Matrix([ [ x1 + y1 for x1, y1 in zip(x,y)] for x, y in zip(self.matrix, other.matrix)])
         return NotImplemented
 
     def __sub__(self, other) -> "Matrix":
-        if isinstance(other, Matrix) and other.cols == self.cols and other.rows == self.rows:
+        if isinstance(other, Matrix):
+            if other.cols != self.cols or other.rows != self.rows:
+                raise NotImplementedError("Matrix dimensions do not match!")
             return Matrix([ [ x1 - y1 for x1, y1 in zip(x,y)] for x, y in zip(self.matrix, other.matrix)])
         return NotImplemented
 
@@ -162,12 +170,14 @@ class Matrix:
     def __mul__(self, other) -> "Matrix":
         if isinstance(other, Matrix):
             return self.multiply(other)
+        if isinstance(other, Number) or type(other) == type(self.matrix[0][0]):
+            return Matrix([ [item * other for item in row] for row in self.matrix ])
         return NotImplemented
 
     def __rmul__(self, other) -> "Matrix":
-        if isinstance(other, Matrix):
-            return self.multiply(other)
-        return Matrix([ [item * other for item in row] for row in self.matrix ])
+        if isinstance(other, Number) or type(other) == type(self.matrix[0][0]):
+             return Matrix([ [item * other for item in row] for row in self.matrix ])
+        return NotImplemented
 
     def rref(self) -> "Matrix":
         "Compute the reduced echelon form of a matrix M."
@@ -242,20 +252,26 @@ class Matrix:
             n, m = self.cols, self.rows
         elif not n:
             n = m
-        zero = 0 * self[0]
+        try:
+            zero = 0 * self[0]
+        except:
+            zero = 0
         return Matrix([[ zero for j in range(n)] for i in range(m) ])
 
     def eye(self, m: int = None, n: int = None):
         "Returns an identity matrix of the same dimension"
         def delta(i, j):
             if i == j:
-                return 1
-            return 0
+                return one
+            return zero
         if not m and not n:
             n, m = self.cols, self.rows
         elif not n:
             n = m
-        zero = 0 * self[0]
+        try:
+            zero = 0 * self[0]
+        except:
+            zero = 0
         one = 1 + zero
         return Matrix([[ delta(i, j) for j in range(n) ] for i in range(m) ])
 
