@@ -2,6 +2,8 @@
 Polynomials
 """
 
+from numbers import Number
+
 class Poly:
     """
     Represents a polynomial as a list of coefficients.
@@ -25,9 +27,18 @@ class Poly:
         if modulus:
             self.mod(modulus)
 
+    def __call__(self, x):
+        return sum(c * x**j for j, c in enumerate(self.coeff))
+
     def __getitem__(self, item):
         return self.coeff[item]
 
+    def __setitem__(self, item, value):
+        self.coeff[item] = value
+
+    def __len__(self):
+        return len(self.coeff)
+
     def __repr__(self):
         def prx(i: int):
             if i == 0:
@@ -37,7 +48,7 @@ class Poly:
             return "x^" + str(i)
 
         if len(self.coeff) == 1:
-            return str(int(self.coeff[0]))
+            return str(self.coeff[0])
         plus = ""
         tmp = ""
         for i in reversed(range(len(self.coeff))):
@@ -76,19 +87,20 @@ class Poly:
         return bool(self.degree()) or bool(self.coeff[0])
 
     def degree(self):
+        "Return the degree."
         return len(self.coeff) - 1
 
     def map(self, func):
+        "Apply a given function to all coefficients."
         self.coeff = list(map(func, self.coeff))
 
     def __add__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
-            try:
-                tmp = self.coeff[:]
+            tmp = self.coeff[:]
+            if isinstance(other, int) or type(other) == type(tmp[0]):
                 tmp[0] += other
                 return self.__class__(tmp, modulus=self.modulus)
-            except:
-                return NotImplemented
+            return NotImplemented
         ls, lo = len(self.coeff), len(other.coeff)
         if ls < lo:
             scoeff = self.coeff + (lo - ls) * [0]
@@ -105,12 +117,10 @@ class Poly:
 
     def __radd__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
-            try:
+            if isinstance(other, Number) or type(other) == type(self.coeff[0]):
                 tmp = self.coeff[:]
                 tmp[0] += other
                 return self.__class__(tmp, modulus=self.modulus)
-            except:
-                pass
         return NotImplemented
 
     def __neg__(self) -> "Poly":
@@ -118,12 +128,11 @@ class Poly:
 
     def __sub__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
-            try:
+            if isinstance(other, int) or type(other) == type(self.coeff[0]):
                 tmp = self.coeff[:]
                 tmp[0] -= other
                 return self.__class__(tmp, modulus=self.modulus)
-            except:
-                return NotImplemented
+            return NotImplemented
         ls, lo = len(self.coeff), len(other.coeff)
         if ls < lo:
             scoeff = self.coeff + (lo - ls) * [0]
@@ -136,24 +145,21 @@ class Poly:
         modulus = self.modulus
         if not modulus and other.modulus:
             modulus = other.modulus
-        return self.__class__([s - o for s, o in zip(scoeff, ocoeff)], modulus=modulus)
+        return self.__class__([s - o for s, o in zip(scoeff, ocoeff)], modulus = modulus)
 
     def __rsub__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
-            try:
+            if isinstance(other, int) or type(other) == type(self.coeff[0]):
                 tmp = self.coeff[:]
                 tmp[0] -= other
                 return self.__class__(tmp, modulus=self.modulus)
-            except:
-                pass
         return NotImplemented
 
     def __mul__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
-            try:
-                return Poly([other * s for s in self.coeff])
-            except:
-                return NotImplemented
+            if isinstance(other, int) or type(other) == type(self.coeff[0]):
+                return Poly([other * s for s in self.coeff], modulus = self.modulus)
+            return NotImplemented
         ls, lo = len(self.coeff), len(other.coeff)
         coeff = [0] * (ls + lo - 1)
         for k in range(ls + lo - 1):
@@ -166,25 +172,38 @@ class Poly:
         modulus = self.modulus
         if not modulus and other.modulus:
             modulus = other.modulus
-        return self.__class__(coeff, modulus=modulus)
+        return self.__class__(coeff, modulus = modulus)
 
     def __rmul__(self, other: int) -> "Poly":
-        return self.__class__([other * s for s in self.coeff], modulus=self.modulus)
+        if isinstance(other, int) or type(other) == type(self.coeff[0]):
+            return self.__class__([other * s for s in self.coeff], modulus=self.modulus)
+        return NotImplemented
 
     def __pow__(self, i: int) -> "Poly":
-        res = self.__class__([1], modulus=self.modulus)
+        if not isinstance(i, int):
+            return NotImplemented
+        zero = 0 * self.coeff[0]
+        one = zero + 1
+        res = self.__class__([one], modulus=self.modulus)
         if i < 0:
             if not self.modulus:
-                raise NotImplementedError(f"Cannot divide.")
+                raise NotImplementedError("Cannot divide polynomials without modulus.")
             tmp = self.inv()
+            i *= -1
         else:
             tmp = self
         for _ in range(i):
             res *= tmp
         return res
 
+    def __floordiv__(self, other: "Poly") -> "Poly":
+        return self.divmod(other)[0]
+
+    def __mod__(self, other: "Poly") -> "Poly":
+        return self.divmod(other)[1]
+
     def divmod(self, other: "Poly") -> ("Poly", "Poly"):
-        "Polynom division with remainder"
+        "Polynom division with remainder."
         if isinstance(other, list):
             other = self.__class__(other)
         elif not isinstance(other, self.__class__):
@@ -215,7 +234,7 @@ class Poly:
         )
 
     def mod(self, other: "Poly") -> None:
-        "Remainder of polynom division"
+        "Reduce with respect to a given polynomial."
         if isinstance(other, list):
             other = self.__class__(other)
         elif not isinstance(other, self.__class__):
@@ -241,6 +260,7 @@ class Poly:
             self.coeff.pop(i)
 
     def inv(self, other: "Poly" = None) -> "Poly":
+        "Inverse modulo a given polynomial."
         if not other:
             other = self.modulus
         if isinstance(other, list):
@@ -249,10 +269,10 @@ class Poly:
             raise NotImplementedError(f"Cannot invert {self} modulo {other}.")
         if not other:
             raise NotImplementedError(f"{other} must be nonzero.")
+        zero = 0 * self.coeff[0]
+        one = zero +1
         r0, r1 = other, self
-        y0, y1 = self.__class__([0], modulus=self.modulus), self.__class__(
-            [1], modulus=self.modulus
-        )
+        y0, y1 = self.__class__([zero], modulus=self.modulus), self.__class__([one], modulus=self.modulus)
         while r1:
             q, r = r0.divmod(r1)
             r0, r1 = r1, r
@@ -260,6 +280,6 @@ class Poly:
         if r0.degree() != 0:
             raise ValueError(f"{self} is not invertible mod {other}.")
         tmp = 1 / r0[0]
-        for i in range(y0.degree()):
+        for i in range(len(y0)):
             y0.coeff[i] *= tmp
         return y0
